#include <QtGui>
#include <QtCore>
// #include <QGLWidget>
#include <QtOpenGL>
#include "Figure3D.h"
#include "math.h"

Figure3D::Figure3D(QWidget* pwgt/*= 0*/) : QGLWidget(pwgt), 
                                            m_xRotate(0), 
                                            m_yRotate(0),
                                            m_zRotate(0),
                                            m_Scale(1)
{ 
//  Figure3D      pwgt 
} 

/*virtual*/void Figure3D::initializeGL() 
{

   qglClearColor(Qt::white);         //     
   glEnable(GL_DEPTH_TEST);          //       ?? 
   glShadeModel(GL_FLAT);            //    
   m_nFigure3D = createFigure3D(1.0f); //    fSize

}

/*virtual*/void Figure3D::resizeGL(int nWidth, int nHeight) 
{ 
   // glViewport(0, 0, (GLint)nWidth, (GLint)nHeight); //        
   
   glViewport((GLint)nWidth/2-(GLint)nHeight/2, 0, (GLint)nHeight, (GLint)nHeight); //  !!!      !!!
   
   glMatrixMode(GL_PROJECTION);                     //     ( ) ?? 
   glLoadIdentity();                                //      ?? 
                                                    //  
   
   // glFrustum(-0.0, 0.0, -0.0, 0.0, 0.0, 0.0);      //   
   
   //  : (, , , , , )
   
   glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
}

/*virtual*/void Figure3D::paintGL() 
{ 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //   

   glMatrixMode(GL_MODELVIEW);                         //      
   glLoadIdentity();                                   //    
//    glTranslatef(0.0, 0.0, -3.0);                       //     
   
   glRotatef(m_xRotate, 1.0, 0.0, 0.0); 
   glRotatef(m_yRotate, 0.0, 1.0, 0.0); 
   glRotatef(m_zRotate, 0.0, 0.0, 1.0);
   
   glScalef(m_Scale*1.0f, m_Scale*1.0f, m_Scale*1.0f);   
   
   glCallList(m_nFigure3D);  //    glCallList       . 

} 

/*virtual*/void Figure3D::mousePressEvent(QMouseEvent* pe) 
{ 
   m_ptPosition = pe->pos(); //       m_ptPosition  
                             //     
} 

/*virtual*/void Figure3D::mouseMoveEvent(QMouseEvent* pe) 
{
   m_xRotate += 180 * (GLfloat) (pe->y() - m_ptPosition.y()) / height(); //   
   m_yRotate += 180 * (GLfloat) (pe->x() - m_ptPosition.x()) / width(); 
      
   updateGL(); //  
   
   m_ptPosition = pe->pos(); 
}

/*virtual*/void Figure3D::keyPressEvent(QKeyEvent* pe)
{
   switch (pe -> key())
   {
         case Qt::Key_Up:    m_Scale = m_Scale*1.02;
         break;
         
         case Qt::Key_Down:  m_Scale = m_Scale/1.02;
         break;
         
         case Qt::Key_Left:  m_zRotate += 2.5;
         break;
         
         case Qt::Key_Right: m_zRotate -= 2.5;
         break;
         
         case Qt::Key_Space:  //    
            m_Scale = 1;
            m_xRotate = 0;
            m_yRotate = 0;
            m_zRotate = 0;
         break;
   }

   updateGL(); //  
}

GLuint Figure3D::createFigure3D(GLfloat fSize/*=1.0f*/) 
{ 
   GLuint n = glGenLists(1); //       
                             //    ??
      
   glNewList(n, GL_COMPILE); 
 
      GLfloat R=0.5*fSize;     
      
      glTranslatef(0.5, 0.0, 0.0); 
      
      //    GLU
      // (  )
      GLUquadricObj *sphere1;
      sphere1 = gluNewQuadric();
      gluQuadricDrawStyle (sphere1, GLU_LINE);   
                 
      glColor4f(0.75f, 0.25f, 0.25f, 1.0f);
      
      glBegin(GL_LINE_STRIP);
      
         // gluSphere (sphere1, R, nLongitudes, nLatitudes);
         gluSphere (sphere1, R, 25, 25);
           
      glEnd();
      
      glTranslatef(-1.0, -0.0, -0.0);
      
      //  
      // (  )
      GLfloat phi;
      GLfloat teta;
      GLfloat pi = 3.141593;
      GLfloat step = pi/25;
      GLfloat eps = 0.001;
      
      glColor4f(0.25f, 0.25f, 0.75f, 1.0f);
      
      GLfloat x1;
      GLfloat y1;
      GLfloat z1;
      
      GLfloat x2;
      GLfloat y2;
      GLfloat z2; 
            
      for (phi=0; phi <= (2*pi+eps); phi += step)
      {
         x1 = 0;
         y1 = R;
         z1 = 0;

         for (teta=0; teta <= (pi+eps); teta += step)
         {
            glBegin(GL_LINE_STRIP);
            
               glVertex3f(x1, y1, z1);
               
               x1=R*sin(teta)*sin(phi);
               y1=R*cos(teta);
               z1=R*sin(teta)*cos(phi);
               
               glVertex3f(x1, y1, z1);
              
               x2=R*sin(teta)*sin(phi+step);
               y2=R*cos(teta);
               z2=R*sin(teta)*cos(phi+step); 
              
               glVertex3f(x2, y2, z2);   
               
               x2=R*sin(teta-step)*sin(phi+step);
               y2=R*cos(teta-step);
               z2=R*sin(teta-step)*cos(phi+step); 
               
               glVertex3f(x2, y2, z2);            
              
            glEnd();
              
          }             
      }
          
   glEndList(); 

return n; 
} 